@@ -17,13 +17,10 @@
 #ifndef GLOBUS_DONT_DOCUMENT_INTERNAL
 /**
  * @file globus_gsi_proxy.c
+ * @brief GSI Proxy Implementation
  * @author Sam Lang, Sam Meder
- *
- * $RCSfile: globus_gsi_proxy.c,v $
- * $Revision: 1.38 $
- * $Date: 2011/01/28 02:21:25 $
  */
-#endif
+#endif /* GLOBUS_DONT_DOCUMENT_INTERNAL */
 
 #define PROXY_NAME                      "proxy"
 #define LIMITED_PROXY_NAME              "limited proxy"
@@ -37,6 +34,15 @@
 
 #ifndef GLOBUS_DONT_DOCUMENT_INTERNAL
 
+#if OPENSSL_VERSION_NUMBER < 0x0090801fL
+#define GT_SK_UNSHIFT_CAST (char *)
+#define GT_I2D_CAST (int (*)())
+#else
+#define GT_SK_UNSHIFT_CAST
+#define GT_I2D_CAST (i2d_of_void *)
+#endif
+
+
 #define GLOBUS_GSI_PROXY_MALLOC_ERROR(_LENGTH_) \
     globus_error_put(globus_error_wrap_errno_error( \
         GLOBUS_GSI_PROXY_MODULE, \
@@ -184,14 +190,12 @@ globus_l_gsi_proxy_deactivate(void)
 #endif /* GLOBUS_DONT_DOCUMENT_INTERNAL */
     
 /**
- * @name Create Request
- */
-/*@{*/
-/**
+ * @brief Create Request
  * @ingroup globus_gsi_proxy_operations
+ * @details
  * Create a proxy credential request
  *
- * This function creates a proxy credential request, ie. a unsigned 
+ * This function creates a proxy credential request, an unsigned 
  * certificate and the corresponding private key, based on the handle
  * that is passed in.
  * The public part of the request is written to the BIO supplied in
@@ -290,7 +294,7 @@ globus_gsi_proxy_create_req(
         GLOBUS_GSI_PROXY_ERROR_RESULT(
             result,
             GLOBUS_GSI_PROXY_ERROR_WITH_X509_REQ,
-            (_PCSL("Could not set version of X509 request in proxy handle")));
+            (_PCSL("Could not set version of X.509 request in proxy handle")));
         goto error_exit;
     }
 
@@ -299,7 +303,7 @@ globus_gsi_proxy_create_req(
         GLOBUS_GSI_PROXY_ERROR_RESULT(
             result,
             GLOBUS_GSI_PROXY_ERROR_WITH_X509_REQ,
-            (_PCSL("Couldn't set public key of X509 request in proxy handle")));
+            (_PCSL("Couldn't set public key of X.509 request in proxy handle")));
         goto error_exit;
     }
 
@@ -599,14 +603,11 @@ globus_gsi_proxy_create_req(
     return result;
 }
 /* globus_gsi_proxy_create_req */
-/*@}*/
 
 /**
- * @name Inquire Request
- */
-/*@{*/
-/**
+ * @brief Inquire Request
  * @ingroup globus_gsi_proxy_operations
+ * @details
  * Inquire a proxy credential request
  *
  * This function reads the public part of a proxy credential request
@@ -808,15 +809,12 @@ globus_gsi_proxy_inquire_req(
     return result;
 }
 /* globus_gsi_proxy_inquire_req */
-/*@}*/
 
 
 /**
- * @name Resign Certificate
- */
-/*@{*/
-/**
+ * @brief Resign Certificate
  * @ingroup globus_gsi_proxy_operations
+ * @details
  * Resign a existing certificate into a proxy
  *
  * This function use the public key in a existing certificate
@@ -972,7 +970,7 @@ globus_gsi_proxy_resign_cert(
         goto done;
     }
 
-    sk_X509_unshift(issuer_cert_chain, issuer_cert);
+    sk_X509_unshift(issuer_cert_chain, GT_SK_UNSHIFT_CAST issuer_cert);
     issuer_cert = NULL;
     
     result = globus_gsi_cred_set_cert_chain(*resigned_credential,
@@ -1006,22 +1004,19 @@ globus_gsi_proxy_resign_cert(
     return result;
 }
 /* globus_gsi_proxy_resign_cert */
-/*@}*/
 
 
 /**
- * @name Sign Request
- */
-/*@{*/
-/**
+ * @brief Sign Request
  * @ingroup globus_gsi_proxy_operations
+ * @details
  * Sign a proxy certificate request
  *
  * This function signs the public part of a proxy credential request,
- * i.e. the unsigned certificate, previously read by inquire req using
- * the supplied issuer_credential. This operation will add a
- * ProxyCertInfo extension to the proxy certificate if values
- * contained in the extension are specified in the handle.
+ * i.e. the unsigned certificate, previously read by
+ * globus_gsi_proxy_inquire_req using the supplied issuer_credential. This
+ * operation will add a ProxyCertInfo extension to the proxy certificate if
+ * values contained in the extension are specified in the handle.
  * The resulting signed certificate is written to the output_bio.
  *
  * @param handle
@@ -1105,7 +1100,7 @@ globus_gsi_proxy_sign_req(
         GLOBUS_GSI_PROXY_OPENSSL_ERROR_RESULT(
             result,
             GLOBUS_GSI_PROXY_ERROR_WITH_X509,
-            (_PCSL("Error converting X509 proxy cert from internal "
+            (_PCSL("Error converting X.509 proxy cert from internal "
              "to DER encoded form")));
         goto done;
     }
@@ -1128,7 +1123,6 @@ globus_gsi_proxy_sign_req(
     return result;
 }
 /* globus_gsi_proxy_sign_req */
-/*@}*/
 
 static globus_result_t
 globus_l_gsi_proxy_sign_key(
@@ -1265,7 +1259,7 @@ globus_l_gsi_proxy_sign_key(
 
         ext_method = X509V3_EXT_get_nid(pci_NID);
 
-        ASN1_digest(i2d_PUBKEY,sha1,(char *) public_key,md,&len);
+        ASN1_digest(GT_I2D_CAST i2d_PUBKEY,sha1,(char *) public_key,md,&len);
 
         sub_hash = md[0] + (md[1] + (md[2] + (md[3] >> 1) * 256) * 256) * 256; 
         
@@ -1358,7 +1352,7 @@ globus_l_gsi_proxy_sign_key(
             GLOBUS_GSI_PROXY_OPENSSL_ERROR_RESULT(
                 result,
                 GLOBUS_GSI_PROXY_ERROR_WITH_X509_EXTENSIONS,
-                (_PCSL("Couldn't create X509 extension list "
+                (_PCSL("Couldn't create X.509 extension list "
                  "to hold PROXYCERTINFO extension")));
             goto done;
         }
@@ -1457,7 +1451,7 @@ globus_l_gsi_proxy_sign_key(
             GLOBUS_GSI_PROXY_OPENSSL_ERROR_RESULT(
                 result,
                 GLOBUS_GSI_PROXY_ERROR_WITH_X509_EXTENSIONS,
-                (_PCSL("Couldn't add X509 extension to new proxy cert")));
+                (_PCSL("Couldn't add X.509 extension to new proxy cert")));
             goto done;
         }
     }
@@ -1489,7 +1483,7 @@ globus_l_gsi_proxy_sign_key(
             GLOBUS_GSI_PROXY_OPENSSL_ERROR_RESULT(
                 result,
                 GLOBUS_GSI_PROXY_ERROR_WITH_X509_EXTENSIONS,
-                (_PCSL("Couldn't get keyUsage extension form issuer cert")));
+                (_PCSL("Couldn't get keyUsage extension from issuer cert")));
             goto done;            
         }
         
@@ -1585,7 +1579,7 @@ globus_l_gsi_proxy_sign_key(
             GLOBUS_GSI_PROXY_OPENSSL_ERROR_RESULT(
                 result,
                 GLOBUS_GSI_PROXY_ERROR_WITH_X509_EXTENSIONS,
-                (_PCSL("Couldn't add X509 keyUsage extension to new proxy cert")));
+                (_PCSL("Couldn't add X.509 keyUsage extension to new proxy cert")));
             X509_EXTENSION_free(extension);
             goto done;
         }
@@ -1621,7 +1615,7 @@ globus_l_gsi_proxy_sign_key(
             GLOBUS_GSI_PROXY_OPENSSL_ERROR_RESULT(
                 result,
                 GLOBUS_GSI_PROXY_ERROR_WITH_X509_EXTENSIONS,
-                (_PCSL("Couldn't add X509 extendedKeyUsage extension to new proxy cert")));
+                (_PCSL("Couldn't add X.509 extendedKeyUsage extension to new proxy cert")));
             goto done;
         }
     }
@@ -1653,7 +1647,7 @@ globus_l_gsi_proxy_sign_key(
                 GLOBUS_GSI_PROXY_OPENSSL_ERROR_RESULT(
                     result,
                     GLOBUS_GSI_PROXY_ERROR_WITH_X509_EXTENSIONS,
-                    (_PCSL("Couldn't add X509 extension to new proxy cert")));
+                    (_PCSL("Couldn't add X.509 extension to new proxy cert")));
                 goto done;
             }        
         }
@@ -1712,7 +1706,7 @@ globus_l_gsi_proxy_sign_key(
         GLOBUS_GSI_PROXY_OPENSSL_ERROR_RESULT(
             result,
             GLOBUS_GSI_PROXY_ERROR_WITH_X509,
-            (_PCSL("Couldn't set pubkey of X509 cert")));
+            (_PCSL("Couldn't set pubkey of X.509 cert")));
         goto done;
     }
 
@@ -1736,7 +1730,7 @@ globus_l_gsi_proxy_sign_key(
         GLOBUS_GSI_PROXY_OPENSSL_ERROR_RESULT(
             result,
             GLOBUS_GSI_PROXY_ERROR_WITH_HANDLE,
-            (_PCSL("The signing algorithm used by the ceritificate cannot "
+            (_PCSL("The signing algorithm used by the certificate cannot "
             " be used to sign a proxy.")));
         goto done;
     }
@@ -1837,11 +1831,9 @@ globus_l_gsi_proxy_sign_key(
 }
 
 /**
- * @name Create Signed
- */
-/* @{ */
-/**
+ * @brief Create Signed
  * @ingroup globus_gsi_proxy_operations
+ * @details
  * Create Signed Proxy Certificate
  *
  * @param handle
@@ -1898,7 +1890,7 @@ globus_gsi_proxy_create_signed(
         }
         result = globus_gsi_proxy_handle_attrs_set_signing_algorithm(
                 handle->attrs,
-                issuer_digest);
+                (EVP_MD *) issuer_digest);
         if (result != GLOBUS_SUCCESS)
         {
             goto exit;
@@ -2081,15 +2073,12 @@ globus_gsi_proxy_create_signed(
     GLOBUS_I_GSI_PROXY_DEBUG_EXIT;
     return result;
 }
-/* @} */            
 
 
 /**
- * @name Assemble credential
- */
-/*@{*/
-/**
+ * @brief Assemble credential
  * @ingroup globus_gsi_proxy_operations
+ * @details
  * Assemble a proxy credential
  *
  * This function assembles a proxy credential. It reads a signed proxy
@@ -2159,7 +2148,7 @@ globus_gsi_proxy_assemble_cred(
         GLOBUS_GSI_PROXY_OPENSSL_ERROR_RESULT(
             result,
             GLOBUS_GSI_PROXY_ERROR_WITH_X509,
-            (_PCSL("Couldn't convert X509 proxy cert from "
+            (_PCSL("Couldn't convert X.509 proxy cert from "
              "DER encoded to internal form")));
         goto done;
     }
@@ -2221,7 +2210,7 @@ globus_gsi_proxy_assemble_cred(
             GLOBUS_GSI_PROXY_OPENSSL_ERROR_RESULT(
                 result,
                 GLOBUS_GSI_PROXY_ERROR_WITH_X509,
-                (_PCSL("Can't read DER encoded X509 cert from BIO")));
+                (_PCSL("Can't read DER encoded X.509 cert from BIO")));
             goto free_cred_handle;
         }
         
@@ -2260,7 +2249,6 @@ globus_gsi_proxy_assemble_cred(
     return result;
 }
 /* globus_gsi_proxy_assemble_cred */
-/*@}*/
 
 /* INTERNAL FUNCTIONS */
 
@@ -2360,7 +2348,7 @@ globus_i_gsi_proxy_set_pc_times(
             GLOBUS_GSI_PROXY_OPENSSL_ERROR_RESULT(
                 result,
                 GLOBUS_GSI_PROXY_ERROR_WITH_X509,
-                (_PCSL("Error adjusting X509 proxy cert's expiration time")));
+                (_PCSL("Error adjusting X.509 proxy cert's expiration time")));
             goto free_pc_notafter;
         }
     }
@@ -2370,7 +2358,7 @@ globus_i_gsi_proxy_set_pc_times(
         GLOBUS_GSI_PROXY_OPENSSL_ERROR_RESULT(
             result,
             GLOBUS_GSI_PROXY_ERROR_WITH_X509,
-            (_PCSL("Error setting X509 proxy cert's expiration")));
+            (_PCSL("Error setting X.509 proxy certificate's expiration")));
         goto free_pc_notafter;
     }
 
@@ -2467,7 +2455,7 @@ globus_i_gsi_proxy_set_subject(
  * Determines the type of proxy to sign based on the requested type and
  * the issuing certificate. Error if we are asked to sign a different format
  * (GSI2 vs GSI3 vs RFC) than the issuer or if the restrictions are invalid
- * (Limited signing non-limited, etc)
+ * (Limited signing non-limited, etc.)
  * In this patch, we also allow support for default proxy types:
  * - if format is unspecified, use issuer format (if that is unspecified, use
  *   RFC)
@@ -2675,7 +2663,7 @@ globus_l_gsi_proxy_determine_type(
         goto exit;
     }
     /**
-     * Finally, verify that we're not trying to use a restriced or independent
+     * Finally, verify that we're not trying to use a restricted or independent
      * proxy with the legacy proxy format
      */
     if (GLOBUS_GSI_CERT_UTILS_IS_GSI_2_PROXY(requested_cert_type) &&
